﻿using System;
using System.Collections.Generic;
using System.Linq;
using EnvelopeCreator.DocusignAPI;
using EnvelopeCreatorProject.DocusignAPI;


namespace EnvelopeCreatorProject.Models.Recipients
{
    internal class SignerRecipientsBuilder: RecipientTypeBuilder
    {
        private readonly List<Tab> _signHereTabs;
        private readonly List<Tab> _initialsTabs;
        private readonly List<Tab> _dataSignFields;
        private readonly List<HtmlTab> _textTabs;
        private readonly List<HtmlTab> _checkboxTabs;
        private readonly List<RadioGroupTab> _radioGroupTabs;


        internal SignerRecipientsBuilder(
            List<Tab> signHereTabs, 
            List<Tab> initialsTabs, 
            List<Tab> dataSignFields,
            List<HtmlTab> textTabs,
            List<HtmlTab> checkboxTabs,
            List<RadioGroupTab> radioGroupTabs)
        {
            _signHereTabs = signHereTabs;
            _initialsTabs = initialsTabs;
            _dataSignFields = dataSignFields;
            _textTabs = textTabs;
            _checkboxTabs = checkboxTabs;
            _radioGroupTabs = radioGroupTabs;
        }

        protected override string GetData(Recipient objRecipient)
        {
            return string.Empty;
        }

        private string GetTabs(List<Tab> tabs, Recipient objRecipient, string name)
        {
            var recipientTabs = tabs.FindAll((Tab tab) => tab.RecipientID == objRecipient.ID);
            return !recipientTabs.Any() ? String.Empty : string.Format("{0}{1}{0}:[{2}]", "\"", name, String.Join(",", recipientTabs.Select(getTabMetaData)));
        }


        private List<T> GetHtmlTabsToInclude<T>(List<T> tabs, string recipientId) where T : Tab {
            return tabs.FindAll((T tab) => tab.RecipientID == recipientId);
        }

        private string GetHtmlTabs(List<HtmlTab> tabs, Recipient objRecipient, string name)
        {
            //This method processes text and checkbox tabs, which can be applied to the current signer, or to anybody
            var tabsToInclude = GetHtmlTabsToInclude(tabs, objRecipient.ID);
            return !tabsToInclude.Any() ? String.Empty : string.Format("{0}{1}{0}:[{2}]", "\"", name, String.Join(",", tabsToInclude.Select(getHtmlTabMetadata)));
        }


        private string GetHtmlRadioGroupTabs(List<RadioGroupTab> tabs, Recipient objRecipient, string name)
        {
            var tabsToInclude = GetHtmlTabsToInclude(tabs, objRecipient.ID);
            return !tabsToInclude.Any() ? String.Empty : string.Format("{0}{1}{0}:[{2}]", "\"", name, String.Join(",", tabsToInclude.Select(getTabMetaData)));
        }


        protected string getTabMetaData (RadioGroupTab tab)
        {
            return string.Format(
                @"{{{0}documentId{0}:{0}{1}{0},
                {0}recipientId{0}:{0}{2}{0},
                {0}groupName{0}:{0}{3}{0},
                {0}required{0}:{4},
                {0}shared{0}:{5},
                {0}radios{0}:[{6}]}}",
                "\"",
                tab.DocumentID,
                tab.RecipientID,
                tab.FieldName,
                tab.Required.ToString().ToLower(),
                tab.Shared.ToString().ToLower(),
                string.Join(",", tab.Radios.Select(getTabMetadata)));
        }

        protected string getTabMetadata(RadioTab tab) {

            return string.Format(
                @"{{{0}xPosition{0}:{0}{1}{0}, 
                {0}yPosition{0}:{0}{2}{0},
                {0}pageNumber{0}:{0}{3}{0},
                {0}selected{0}:{0}{4}{0},
                {0}value{0}:{0}{5}{0},
                {0}required{0}:{0}{6}{0}}}",
                "\"",
                tab.XPosition,
                tab.YPosition,
                tab.PageNumber,
                tab.Selected.ToString().ToLower(),
                tab.Value,
                tab.Required.ToString().ToLower());
        }


        protected string getHtmlTabMetadata(HtmlTab tab)
        {

            return string.Format(
                @"{{{0}xPosition{0}:{0}{1}{0},
                {0}yPosition{0}:{0}{2}{0}, 
                {0}documentId{0}:{0}{3}{0},
                {0}pageNumber{0}:{0}{4}{0},
                {0}recipientId{0}:{0}{5}{0},
                {0}tabLabel{0}:{0}{6}{0},
                {0}value{0}:{0}{7}{0},
                {0}locked{0}:{8},
                {0}required{0}:{9},
                {0}shared{0}:{10},
                {0}selected{0}:{11},
                {0}fontColor{0}:{0}{12}{0},
                {0}width{0}:{0}{13}{0}}}",
                    "\"",
                    tab.XPosition,
                    tab.YPosition,
                    tab.DocumentID,
                    tab.PageNumber,
                    tab.RecipientID,
                    tab.TabLabel,
                    tab.Value,
                    tab.Locked.ToString().ToLower(),
                    tab.Required.ToString().ToLower(),
                    tab.Shared.ToString().ToLower(),
                    tab.Selected.ToString().ToLower(),
                    tab.FontColor.ToString(),
                    tab.Width);
        }

        protected string getTabMetaData(Tab tab)
        {
            return string.Format(
                @"{{{0}xPosition{0}:{0}{1}{0},
                {0}yPosition{0}:{0}{2}{0}, 
                {0}documentId{0}:{0}{3}{0},
                {0}pageNumber{0}:{0}{4}{0},
                {0}recipientId{0}:{0}{5}{0},
                {0}optional{0}:{6}}}",
                "\"",
                tab.XPosition,
                tab.YPosition,
                tab.DocumentID,
                tab.PageNumber,
                tab.RecipientID,
                tab.Optional.ToString().ToLower());
        }

        protected string GetSignatureInfoTo(Recipient objRecipient)
        {
            if (objRecipient.SignatureInfo == null)
                return string.Empty;

            return string.Format(
                 "{0}signatureInfo{0}:{{{0}fontStyle{0}:{0}{1}{0},{0}signatureName{0}:{0}{2}{0},{0}signatureInitials{0}:{0}{3}{0}}}",
                 "\"",
                 objRecipient.SignatureInfo.FontStyle.ToString(),
                 objRecipient.SignatureInfo.SignatureName,
                 objRecipient.SignatureInfo.SignatureInitials
                 );
        }

        protected string GetSignTabs(Recipient objRecipient)
        {
            return string.Format("{0}tabs{0}:{{{1}}}",
                "\"",
                GetTabSection(
                    GetTabs(_signHereTabs, objRecipient, "signHereTabs"), //SignHere Tabs (FullName as signature)
                    GetTabs(_initialsTabs, objRecipient, "initialHereTabs"), //SignInitials Tabs
                    GetTabs(_dataSignFields, objRecipient, "dateSignedTabs"), //Date Signed Tabs
                    GetHtmlTabs(_textTabs, objRecipient, "textTabs"), //Textbox Tabs
                    GetHtmlTabs(_checkboxTabs, objRecipient, "checkboxTabs"), //Checkbox Tabs
                    GetHtmlRadioGroupTabs(_radioGroupTabs, objRecipient, "radioGroupTabs") //Radio buttons Tabs
                )
            );
        }

        private string GetTabSection(params string[] tabs)
        {
            return string.Join(",", tabs.AsEnumerable().Where(t => !string.IsNullOrEmpty(t)));
        }
    }
}
