using EnvelopeCreatorProject;
using EnvelopeCreatorProject.Models.Signers;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using quikesigntransport.selfservice.Models;
using quikesigntransport.selfservice.Services;

namespace quikesigntransport.selfservice
{
    public class Startup
    {
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;

            Settings = configuration.GetSection("SelfServiceSettings")
                                    .Get<SelfServiceSettings>(c => c.BindNonPublicProperties = true);

        }

        public IConfiguration Configuration { get; }

        public SelfServiceSettings Settings { get; } 

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddControllers();

            services.AddScoped<ISignHelperService, SignHelperService>(
                s => new SignHelperService(
                    Settings.QUIK_USERNAME, 
                    Settings.QUIK_PASSWORD,
                    Settings.QUIK_OAUTHTOKEN_ENDPOINT,
                    Settings.QUIK_DOCUSIGN_SELFSERVICE_ENDPOINT,
                    Settings.DOCUSIGN_AUTHTOKEN,
                    Settings.DOCUSIGN_ENVIRONMENT_URL,
                    Settings.DocusignIntegratorKey, 
                    Settings.DocusignAPIVersion,
                    s.GetService<ILogger<OAuthSigner>>(),
                    s.GetService<ILogger<DocusignTokenHandler>>())
                );
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseHttpsRedirection();

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
